/*******************************************************************************
 * Copyright (c) 2005 IBM Corporation and others. All rights reserved. This
 * program and the accompanying materials are made available under the terms of
 * the Eclipse Public License v1.0 which accompanies this distribution, and is
 * available at http://www.eclipse.org/legal/epl-v10.html $Id:
 * DeleteFileCommand.java,v 1.1 2005/05/03 14:32:54 sschneid Exp $
 * 
 * Contributors: IBM - Initial API and implementation
 ******************************************************************************/

package org.eclipse.hyades.internal.execution.core.file;

import java.io.File;
import java.io.IOException;
import java.util.Iterator;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.hyades.execution.core.file.IFileManagerExtended.Cookie;
import org.eclipse.hyades.execution.core.file.IFileManagerExtended.FileIdentifierList;

class DeleteFileCommand extends AbstractFileManipulationCommand implements IDeleteFileCommand {

    /**
     * The client side personality of the delete command
     * 
     * @author Scott E. Schneider
     */
    private class Client extends AbstractFileManipulationCommand.Client {

        /**
         * Creates the client side personality for the delete command
         * 
         * @param channel
         *            the channel to communicate with the server
         */
        Client(ISocketChannel channel) {
            super(channel);
        }

        /*
         * (non-Javadoc)
         * 
         * @see org.eclipse.hyades.internal.execution.core.file.ICommand#execute()
         */
        public void execute() throws IOException {

            // Initiate client
            super.execute();

            // Send the number of files to be removed
            FileIdentifierList remoteIdentifiers = DeleteFileCommand.this.remoteIdentifiers;
            this.send(remoteIdentifiers.size());

            // Send all remote identifier names
            this.send(remoteIdentifiers.getArray());

        }
    }

    /**
     * The server side personality of the delete command
     * 
     * @author Scott E. Schneider
     */
    private class Server extends AbstractFileManipulationCommand.Server {

        /**
         * Constructs the server side state personality for the delete command
         * 
         * @param channel
         *            the channel to communicate with the client
         */
        Server(ISocketChannel channel) {
            super(channel);
        }

        /*
         * (non-Javadoc)
         * 
         * @see org.eclipse.hyades.internal.execution.core.file.ICommand#execute()
         */
        public void execute() throws IOException {

            // Initiate server
            super.execute();

            // Receive number of files that will be deleted
            int count = this.receiveInt();

            // Receive the remote identifiers from the client
            FileIdentifierList remoteIdentifiers = FileIdentifierList.create(this.receiveStrings());
            DeleteFileCommand.this.remoteIdentifiers = remoteIdentifiers;

            // Remove all the files on the server side
            for (Iterator identifiers = remoteIdentifiers.iterator(); identifiers.hasNext();) {
                String fileName = (String) identifiers.next();
                File remoteFile = new File(fileName);
                if (remoteFile.exists()) {
                    remoteFile.delete();
                }
            }

        }
    }

    DeleteFileCommand(Cookie cookie, IProgressMonitor monitor) {
        super(DeleteFileCommand.class, cookie, monitor);
    }

    public DeleteFileCommand(ISocketChannel channel) {
        super(DeleteFileCommand.class);
        this.setState(new Server(channel));
    }

    DeleteFileCommand(ISocketChannel channel, FileIdentifierList remoteIdentifiers, IProgressMonitor monitor) {
        super(DeleteFileCommand.class, remoteIdentifiers, monitor);
        this.setState(new Client(channel));
    }

}