/**********************************************************************
 * Copyright (c) 2005 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: SensorSection.java,v 1.13 2005/03/30 16:21:17 ewchan Exp $
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/

package org.eclipse.hyades.logging.adapter.ui.internal.util;

import java.math.BigInteger;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.edit.provider.IItemPropertyDescriptor;
import org.eclipse.emf.edit.provider.INotifyChangedListener;
import org.eclipse.hyades.logging.adapter.model.internal.configuration.ContextInstanceType;
import org.eclipse.hyades.logging.adapter.model.internal.sensor.SensorConfigType;
import org.eclipse.hyades.logging.adapter.model.internal.sensor.SensorType;
import org.eclipse.hyades.logging.adapter.model.internal.sensor.provider.SensorConfigTypeItemProvider;
import org.eclipse.hyades.logging.adapter.model.internal.sensor.provider.SensorItemProviderAdapterFactory;
import org.eclipse.hyades.logging.adapter.ui.AcadEditorPlugin;
import org.eclipse.hyades.logging.adapter.ui.internal.views.AcadGuiModelManager;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CCombo;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

public class SensorSection extends Composite
								implements IDetails, ModifyListener, SelectionListener
										   , INotifyChangedListener
{
	protected CCombo typeCombo;
	protected Label uniqueIDText;		
	protected Text descriptionText;		
	
	protected Text maxText;
	protected Text fileFooterSizeText;
	protected Text confidenceBufferSizeText;
	
    protected boolean isDisplaying;
    protected SensorConfigType currentSelection;
    protected SensorConfigTypeItemProvider itemProvider;
    protected SensorItemProviderAdapterFactory itemFactory;
	
	
	public SensorSection(RuleBuilderWidgetFactory factory, Composite composite, int style){

		super(composite, style);
		
		itemFactory = new SensorItemProviderAdapterFactory();

		GridLayout detailLayout = new GridLayout();
		detailLayout.numColumns = 2;
		detailLayout.verticalSpacing = 5;

		this.setLayout(detailLayout);
		this.setLayoutData(GridUtil.createFill());
						
		Label label = factory.createLabel(this, AcadEditorPlugin.getPlugin().getString("STR_Sensor_Type"), SWT.NONE);	
		typeCombo = factory.createCombo(this, SWT.READ_ONLY);
		typeCombo.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		typeCombo.setToolTipText(AcadEditorPlugin.getPlugin().getString("SINCE_1_3"));
		initializeTypeCombo();	

		label = factory.createLabel(this, AcadEditorPlugin.getPlugin().getString("STR_SENSOR_DESCR"), SWT.NONE);	
		descriptionText = factory.createText(this, "", SWT.NULL);
		descriptionText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));		

		label = factory.createLabel(this, AcadEditorPlugin.getPlugin().getString("STR_SENSOR_ID"), SWT.NONE);	
		uniqueIDText = factory.createLabel(this, "", SWT.NULL);
		uniqueIDText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));		

		label = factory.createLabel(this, "", SWT.NONE);	
		GridData data = new GridData(GridData.FILL_HORIZONTAL);
		data.horizontalSpan = 2;
		label.setLayoutData(data);		

		label = factory.createLabel(this, AcadEditorPlugin.getPlugin().getString("STR_SENSOR_MAXB"), SWT.NONE);	
		maxText = factory.createText(this, "", SWT.NULL);
		maxText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		label = factory.createLabel(this, AcadEditorPlugin.getPlugin().getString("STR_SENSOR_CONFB"), SWT.NONE);	
		confidenceBufferSizeText = factory.createText(this, "", SWT.NULL);
		confidenceBufferSizeText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));		
		
		label = factory.createLabel(this, AcadEditorPlugin.getPlugin().getString("STR_SENSOR_FOOTB"), SWT.NONE);	
		fileFooterSizeText = factory.createText(this, "", SWT.NULL);
		fileFooterSizeText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));		
		
		// add listeners - don't change the order in which the listeners are added
		// because of team support issues modify listeners should always be the last one added
		maxText.addModifyListener(this);
		confidenceBufferSizeText.addModifyListener(this);
		fileFooterSizeText.addModifyListener(this);
		descriptionText.addModifyListener(this);
		typeCombo.addSelectionListener(this);
				
		itemProvider = (SensorConfigTypeItemProvider)itemFactory.createSensorConfigTypeAdapter();
		itemProvider.addListener(this);
	}
	
	/**
	 * @see IDetails#display(Object)
	 */
	public void display(EObject object) {
        
		if(object != null)
			AcadGuiModelManager.setCurrentModel((ContextInstanceType)object.eContainer());
        
		if(currentSelection == object)
		  return;
        
		if(maxText.isFocusControl()
		   || confidenceBufferSizeText.isFocusControl()
		   || fileFooterSizeText.isFocusControl()
		   || uniqueIDText.isFocusControl()
		   || descriptionText.isFocusControl())
		   
		  return;
        
		currentSelection = (SensorConfigType)object;
		isDisplaying = true;
		
		maxText.setText(currentSelection.getMaximumBlocking()==null?"":currentSelection.getMaximumBlocking());
		confidenceBufferSizeText.setText(currentSelection.getConfidenceBufferSize()==null?"":""+currentSelection.getConfidenceBufferSize());
		fileFooterSizeText.setText(currentSelection.getFileFooterSize()==null?"":""+currentSelection.getFileFooterSize());
		descriptionText.setText(currentSelection.getDescription()==null?"":currentSelection.getDescription());
		uniqueIDText.setText(currentSelection.getUniqueID()==null?"":currentSelection.getUniqueID());
		typeCombo.select(SensorType.VALUES.indexOf(currentSelection.getType()));
		
		isDisplaying = false;
	}

	/**
	 * @see IDetails#updateCurrentSelection(Object)
	 */
	public boolean updateCurrentSelection(EObject currentSelection) {

 		this.currentSelection = (SensorConfigType)currentSelection;
		
		this.currentSelection.setUniqueID(uniqueIDText.getText());
		this.currentSelection.setDescription(descriptionText.getText());
		this.currentSelection.setMaximumBlocking(maxText.getText());
		this.currentSelection.setConfidenceBufferSize(new BigInteger(confidenceBufferSizeText.getText()));
		this.currentSelection.setFileFooterSize(new BigInteger(fileFooterSizeText.getText()));
				
		return false;

	}
	
	public void setFocusToText(){
	        descriptionText.setFocus();
	        descriptionText.selectAll();
	}

	/**
	 * Sent when the text is modified.
	 *
	 * @param e an event containing information about the modify
	 */
	public void modifyText(ModifyEvent e)
	{
		
		if(isDisplaying || currentSelection == null)
		  return;
		  
		if(e.getSource() == maxText)
		{
			String text = maxText.getText();
			if(!text.equals(currentSelection.getMaximumBlocking()))
			{
				if(text.length() == 0 && currentSelection.getMaximumBlocking() == null)
				  return;
				  
				IItemPropertyDescriptor descriptor = 
					itemProvider.getPropertyDescriptor(
						currentSelection,
						SensorConfigTypeItemProvider.MAX_BLOCKING);
						
				if(descriptor != null)
					descriptor.setPropertyValue(currentSelection, text);				
			}
		}
		else if(e.getSource() == confidenceBufferSizeText)
		{
			String text = confidenceBufferSizeText.getText();
			if(!text.equals(""+currentSelection.getConfidenceBufferSize()))
			{
				if(text.length() == 0 && currentSelection.getConfidenceBufferSize() == null)
				  return;
				  
				IItemPropertyDescriptor descriptor = 
					itemProvider.getPropertyDescriptor(
						currentSelection,
						SensorConfigTypeItemProvider.CONFIDENCE_BUFFER);
						
				if(descriptor != null)
					descriptor.setPropertyValue(currentSelection, new BigInteger(text));				
			}
		}
		else if(e.getSource() == fileFooterSizeText)
		{
			String text = fileFooterSizeText.getText();
			if(!text.equals(""+currentSelection.getFileFooterSize()))
			{
				if(text.length() == 0 && currentSelection.getFileFooterSize() == null)
				  return;
				  
				IItemPropertyDescriptor descriptor = 
					itemProvider.getPropertyDescriptor(
						currentSelection,
						SensorConfigTypeItemProvider.FILE_FOOTER_BUFFER);
						
				if(descriptor != null)
					descriptor.setPropertyValue(currentSelection, new BigInteger(text));				
			}
		}
		else if(e.getSource() == descriptionText)
		{
			String text = descriptionText.getText();
			if(!text.equals(currentSelection.getDescription()))
			{
				if(text.length() == 0 && currentSelection.getDescription() == null)
				  return;
				  
				IItemPropertyDescriptor descriptor = 
					itemProvider.getPropertyDescriptor(
						currentSelection,
						SensorConfigTypeItemProvider.DESCRIPTION);
						
				if(descriptor != null)
					descriptor.setPropertyValue(currentSelection, text);				
			}
		}
	}

	public void dispose() {
		
		itemFactory = null;
		itemProvider = null;
	}

	/**
	 */
	public void notifyChanged(Notification notification)
	{
		Object obj = notification.getNotifier();
		
		if(obj == null || (obj != currentSelection))
		  return;
		  
		display(currentSelection);
	}
	
	public void widgetDefaultSelected(SelectionEvent e) {
	}
	
	public void widgetSelected(SelectionEvent e)
	{
		if (e.getSource() == typeCombo) {
			
			String sel = typeCombo.getText();
			if (!sel.equals(currentSelection.getType().getName())) {
			
				IItemPropertyDescriptor descriptor = itemProvider
				.getPropertyDescriptor(currentSelection,
						SensorConfigTypeItemProvider.TYPE);
				
				if(descriptor != null)
					descriptor.setPropertyValue(currentSelection, SensorType.get(sel));
				
				currentSelection.setType(SensorType.get(sel));
			}
		}
		
	}
	/**
	 * 
	 */
	protected void initializeTypeCombo() {
		for (int i = 0; i < SensorType.VALUES.size(); i++) {
			typeCombo.add(((SensorType)SensorType.VALUES.get(i)).getName());
		}
	}

}
