/*******************************************************************************
 * Copyright (c) 2005 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: EObjectTreeSection.java,v 1.8 2005/03/15 14:52:11 dguilbaud Exp $
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.hyades.test.ui.internal.editor.form.util;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.naming.ldap.Control;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.emf.common.notify.impl.AdapterImpl;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.hyades.test.ui.adapter.TestWorkbenchAdapter;
import org.eclipse.hyades.test.ui.editor.form.util.EditorForm;
import org.eclipse.hyades.test.ui.internal.model.EMFUtil;
import org.eclipse.hyades.test.ui.internal.model.ui.MoveChildrenDownAction;
import org.eclipse.hyades.test.ui.internal.model.ui.MoveChildrenUpAction;
import org.eclipse.hyades.test.ui.internal.model.ui.RemoveChildrenAction;
import org.eclipse.hyades.ui.HyadesUIPlugin;
import org.eclipse.hyades.ui.internal.action.CollapseAllAction;
import org.eclipse.hyades.ui.internal.provider.WorkbenchAdapterLabelProvider;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.viewers.IContentProvider;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.StructuredViewer;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.TreeItem;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.actions.SelectionListenerAction;
import org.eclipse.ui.part.DrillDownAdapter;

/**
 * @author marcelop
 * @since 
 */
public class EObjectTreeSection 
extends TreeSection
{
	public static class EObjectContainerContentProvider
	extends AdapterImpl implements ITreeContentProvider
	{
		private EStructuralFeature eStructuralFeature;
		private EObject container;
		private Viewer viewer;
		private EditorForm editorForm;
		private Map eStructuralFeatureByClass;
		
		/**
		 * Constructor for EObjectContainerContentProvider
		 * @param editorForm
		 * @param eStructuralFeature
		 */
		public EObjectContainerContentProvider(EditorForm editorForm, EStructuralFeature eStructuralFeature)
		{
			this.eStructuralFeature = eStructuralFeature;
			this.editorForm = editorForm;
		}
		
		/**
		 * @see org.eclipse.jface.viewers.IContentProvider#dispose()
		 */
		public void dispose()
		{
			if(container != null)
			{
				container.eAdapters().remove(this);
				Object value = container.eGet(eStructuralFeature);
				if(value instanceof Collection)
					EMFUtil.removeAdapter(((Collection)value).iterator(), this);
				else if(value != null)
					((Notifier)value).eAdapters().remove(this);

				container = null;
			}
			
			viewer = null;
			eStructuralFeature = null;
			editorForm = null;
			
			if(eStructuralFeatureByClass != null)
				eStructuralFeatureByClass.clear();
		}
		
		protected EObject getContainer()
		{
			return container;
		}
		
		public void registerParent(Class cls, EStructuralFeature eStructuralFeature)
		{
			if(eStructuralFeatureByClass == null)
				eStructuralFeatureByClass = new HashMap();
			eStructuralFeatureByClass.put(cls, eStructuralFeature);
		}	

		public void deregisterParent(Class cls)
		{
			if(eStructuralFeatureByClass != null)
				eStructuralFeatureByClass.remove(cls);
		}	
		
		protected Object getRegisteredParentChild(Object parent)
		{
			if(eStructuralFeatureByClass != null)
			{
				for (Iterator i = eStructuralFeatureByClass.entrySet().iterator(); i.hasNext();)
				{
					Map.Entry entry = (Map.Entry)i.next();
					if(((Class)entry.getKey()).isInstance(parent))
					{
						if(parent instanceof EObject)
							return ((EObject)parent).eGet((EStructuralFeature)entry.getValue());
						return null;						
					}
				}
			}
			return null;						
		} 

		/**
		 * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer, java.lang.Object, java.lang.Object)
		 */
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
		{
			this.viewer = viewer;
			
			if(container != null)
			{
				container.eAdapters().remove(this);
				Object value = container.eGet(eStructuralFeature);
				deregisterInputAdapters(container, value);
				container = null;
			}
			
			if(newInput instanceof EObject)
			{
				EObject eObject = (EObject)newInput;
				if(eStructuralFeature.eContainer() == eObject.eClass())
				{
					container = eObject;
					container.eAdapters().add(this);
					registerInputAdapters(eObject, container.eGet(eStructuralFeature));
				}
			}
		}
		
		protected Viewer getViewer()
		{
			return viewer;
		}
		
		protected void deregisterInputAdapters(EObject oldInput, Object value)
		{
			if(value instanceof Collection)
				EMFUtil.removeAdapter(((Collection)value).iterator(), this);
			else if(value != null)
				((Notifier)value).eAdapters().remove(this);			
		}
		
		protected void registerInputAdapters(EObject newInput, Object value)
		{
			if(value instanceof List)
			{	
				EMFUtil.addAdapter(((List)value).iterator(), this);
			}
			else if(value != null)
			{
				((Notifier)value).eAdapters().add(this);
			}			
		}

		/**
		 * @see org.eclipse.jface.viewers.ITreeContentProvider#getParent(java.lang.Object)
		 */
		public Object getParent(Object element)
		{
			if(container == element)
				return null;
				
			if(element instanceof EObject)
				return ((EObject)element).eContainer();
			
			return null;
		}
		
		/**
		 * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
		 */
		public Object[] getElements(Object inputElement)
		{
			if(container == null)
				return new Object[0];

			if(eStructuralFeature.isMany())
				return ((List)container.eGet(eStructuralFeature)).toArray();

			return new Object[]{container.eGet(eStructuralFeature)}; 
		}
		

		/**
		 * @see org.eclipse.jface.viewers.ITreeContentProvider#hasChildren(java.lang.Object)
		 */
		public boolean hasChildren(Object element)
		{
			if(container == null)
				return false;

			if(!eStructuralFeature.isMany())
				return false;

			if(container == element)
			{
				List values = (List)container.eGet(eStructuralFeature);
				return !values.isEmpty();
			}
			
			Object child = getRegisteredParentChild(element);
			return (child != null);
		}

		/**
		 * @see org.eclipse.jface.viewers.ITreeContentProvider#getChildren(java.lang.Object)
		 */
		public Object[] getChildren(Object parentElement)
		{
			Object child = getRegisteredParentChild(parentElement);
			if(child != null)
			{
				if(child instanceof Collection)
					return ((Collection)child).toArray();
				return new Object[]{child};
			}
			
			return new Object[0];
		}

		/**
		 * @see org.eclipse.emf.common.notify.Adapter#notifyChanged(org.eclipse.emf.common.notify.Notification)
		 */
		public void notifyChanged(Notification msg)
		{
			switch(msg.getEventType())
			{
				case Notification.ADD:
				case Notification.ADD_MANY:
				case Notification.REMOVE:
				case Notification.REMOVE_MANY:
				case Notification.MOVE:
				case Notification.SET:
				case Notification.UNSET:
					break;
					
				default:
					return; 	
			}
			
			editorForm.getBaseEditorExtension().markDirty();
			adjustModel(msg);
			adjustAdapter(msg);
			updateViewer(msg);
		}
		
		protected void adjustModel(Notification msg)
		{
		}
		
		protected boolean shouldAdjustAdapter(Notification msg)
		{
			return ((msg.getNotifier() == container) && (msg.getFeature() == eStructuralFeature)); 
		}
		
		protected void adjustAdapter(Notification msg)
		{
			if(shouldAdjustAdapter(msg))
			{
				switch(msg.getEventType())
				{
					case Notification.ADD:
						((Notifier)msg.getNewValue()).eAdapters().add(this);
						break;

					case Notification.ADD_MANY:
						EMFUtil.addAdapter(((Collection)msg.getNewValue()).iterator(), this);
						break;
						
					case Notification.REMOVE:
						((Notifier)msg.getOldValue()).eAdapters().remove(this);
						break;

					case Notification.REMOVE_MANY:
						EMFUtil.removeAdapter(((Collection)msg.getOldValue()).iterator(), this);
						break;
				}
			}
			
		}

		protected boolean shouldUpdateViewer(Notification msg)
		{
			return (msg.getFeature() == eStructuralFeature);	
		}

		protected void updateViewer(Notification msg)
		{
			if(getViewer() == null)
				return;

			if(!(getViewer() instanceof StructuredViewer))
			{
				getViewer().refresh();
				return;
			}

			if(msg.getNotifier() != container)
			{
				switch(msg.getEventType())
				{
					case Notification.SET:
					case Notification.UNSET:
						attributeChanged(msg);
						return;
				}
			}

			if(!(getViewer() instanceof TreeViewer))
			{
				getViewer().refresh();
				return;
			}
				
			if(shouldUpdateViewer(msg))
			{	
				Object parent = null;
				switch(msg.getEventType())
				{
					case Notification.ADD:
						parent = getParent(msg.getNewValue());
						if(parent == null)
							parent = getViewer().getInput();
						((TreeViewer)getViewer()).add(parent, msg.getNewValue());
						getViewer().setSelection(new StructuredSelection(msg.getNewValue()), true);
						break;
	
					case Notification.ADD_MANY:
						Object[] addedObjects = ((Collection)msg.getNewValue()).toArray();
						parent = getParent(addedObjects[0]);
						if(parent == null)
							parent = getViewer().getInput();
							
						((TreeViewer)getViewer()).add(parent, addedObjects);
						getViewer().setSelection(new StructuredSelection(addedObjects), true);
						break;
							
					case Notification.REMOVE:
						((TreeViewer)getViewer()).remove(msg.getOldValue());
						Object value = ((EObject)msg.getNotifier()).eGet((EStructuralFeature)msg.getFeature());
						if(value instanceof List)
						{
							List values = (List)value;
							if(values.isEmpty())
							{
								getViewer().getControl().setFocus();
							}
							else
							{
								int position = msg.getPosition();
								position = (position<values.size())?position:(values.size()-1);
								getViewer().setSelection(new StructuredSelection(values.get(position)), true);
							}
						}
						else
						{
							getViewer().getControl().setFocus();
						}
						break;
	
					case Notification.REMOVE_MANY:
						((TreeViewer)getViewer()).remove(((Collection)msg.getOldValue()).toArray());
						getViewer().getControl().setFocus();
						break;
	
					case Notification.MOVE:
						getViewer().refresh();
						break;
				}
			}
		}
		
		protected void attributeChanged(Notification msg)
		{
			((StructuredViewer)getViewer()).update(msg.getNotifier(), null);
		}
	}
			
	private EStructuralFeature eStructuralFeature;
	
	protected DrillDownAdapter drillDownAdapter;
	protected CollapseAllAction collapseAllAction;
	
	protected IAction addAction;
	protected IAction insertAction;
	protected RemoveChildrenAction removeAction;
	protected MoveChildrenUpAction moveUpAction;
	protected MoveChildrenDownAction moveDownAction;
	
	/**
	 * Constructor for EObjectTreeSection
	 * @param editorForm
	 * @param buttonLabels
	 */
	public EObjectTreeSection(EditorForm editorForm, EStructuralFeature eStructuralFeature, IAction addAction)
	{
		super(editorForm);
		this.eStructuralFeature = eStructuralFeature;
		this.addAction = addAction;
		
		String name = null;
		if(addAction != null)
			name = addAction.getText();
		
		adjustButtonLabels(name);		
	}

	/**
	 * @see org.eclipse.update.ui.forms.internal.FormSection#dispose()
	 */
	public void dispose()
	{
		eStructuralFeature = null;
		
		if(drillDownAdapter != null)
		{
			getTreeViewer().removeSelectionChangedListener(drillDownAdapter);
			drillDownAdapter = null;
		}
		
		if(collapseAllAction != null)
		{
			collapseAllAction.dispose();
			collapseAllAction = null;
		}
		if(removeAction != null)
		{
			removeAction.dispose();
			removeAction = null;
		}
		if(moveUpAction != null)
		{
			moveUpAction.dispose();
			moveUpAction = null;
		}
		if(moveDownAction != null)
		{
			moveDownAction.dispose();
			moveDownAction = null;
		}
		
		addAction = null;
		insertAction = null;
		super.dispose();
	}
	
	protected EStructuralFeature getEStructuralFeature()
	{
		return eStructuralFeature;
	}
		
	protected void adjustButtonLabels(String addLabel)
	{
		removeAction = new RemoveChildrenAction(false);
		moveUpAction = new MoveChildrenUpAction();
		moveDownAction = new MoveChildrenDownAction();
				
		setButtonLabels(new String[]{addLabel, removeAction.getText(), moveUpAction.getText(), moveDownAction.getText()});		
	}
	
	/**
	 * This method allows the add action to be added after the constructor time so there 
	 * won't be any button for it.
	 * @param addAction
	 */
	protected void adjustButtonLabels(String addLabel, String insertLabel)
	{
		removeAction = new RemoveChildrenAction(false);
		moveUpAction = new MoveChildrenUpAction();
		moveDownAction = new MoveChildrenDownAction();
				
		setButtonLabels(new String[]{addLabel, insertLabel, removeAction.getText(), moveUpAction.getText(), moveDownAction.getText()},new String[]{HyadesUIPlugin.getString("TOOLTIP_ADD"),HyadesUIPlugin.getString("TOOLTIP_INSERT"),HyadesUIPlugin.getString("TOOLTIP_REMOVE"),HyadesUIPlugin.getString("TOOLTIP_UP"),HyadesUIPlugin.getString("TOOLTIP_DOWN")});		    //$NON-NLS-1$//$NON-NLS-2$//$NON-NLS-3$//$NON-NLS-4$ //$NON-NLS-5$
	}

	
	/**
	 * This method allows the add action to be added after the constructor time so there 
	 * won't be any button for it.
	 * @param addAction
	 */
	protected void setAddAction(IAction addAction)
	{
		this.addAction = addAction;
	}

	/**
	 * @see org.eclipse.hyades.test.ui.internal.editor.form.base.BaseTreeSection#adjustClient(org.eclipse.swt.widgets.Composite)
	 */
	protected void adjustClient(Composite parent)
	{
		getTreeViewer().setContentProvider(createContentProvider());
		getTreeViewer().setLabelProvider(createLabelProvider());
		getTreeViewer().setAutoExpandLevel(0);

		if(addAction != null)
			getTreePart().setButtonEnabled(0, addAction.isEnabled());
		
		if(insertAction != null)
			getTreePart().setButtonEnabled(1, insertAction.isEnabled());

		if(moveDownAction != null)
			moveDownAction.setStructuredViewer(getTreeViewer());
		if(moveUpAction != null)
			moveUpAction.setStructuredViewer(getTreeViewer());
		if(removeAction != null)
			removeAction.setStructuredViewer(getTreeViewer());
	}
	
	/**
	 * Creates this section's content provider
	 * @return IContentProvider
	 */
	protected IContentProvider createContentProvider()
	{
		return new EObjectContainerContentProvider(getEditorForm(), getEStructuralFeature());
	}

	/**
	 * Creates this section's label provider
	 * @return ILabelProvider
	 */
	protected ILabelProvider createLabelProvider()
	{
		return new WorkbenchAdapterLabelProvider(TestWorkbenchAdapter.class);
	}

	/**
	 * @see org.eclipse.hyades.test.ui.internal.editor.form.base.BaseStructuredViewerSection#createActions()
	 */
	protected void createActions()
	{
		collapseAllAction = new CollapseAllAction(getTreeViewer()).useDefaultDecorators();
		
		drillDownAdapter = new DrillDownAdapter(getTreeViewer());
		getTreeViewer().addSelectionChangedListener(drillDownAdapter);

		if(removeAction != null)
			removeAction.setEnabled(false);

		if(moveDownAction != null)
		{
			moveDownAction.setEStructuralFeature(getEStructuralFeature());
			moveDownAction.setEnabled(false);
		}		

		if(moveUpAction != null)
		{
			moveUpAction.setEStructuralFeature(getEStructuralFeature());
			moveUpAction.setEnabled(false);
		}
	}

	/**
	 * @see org.eclipse.hyades.test.ui.internal.editor.form.base.BaseStructuredViewerSection#buttonSelected(int)
	 */
	protected void buttonSelected(int index)
	{
		if(!isReadOnly())
		{
			switch(index)
			{
				case 0:
					doAdd();
					break;
				
				case 1:
					doRemove();
					break;
	
				case 2:
					doMoveUp();
					break;
	
				case 3:
					doMoveDown();
					break;
				
			}
		}
		
		updateActionsAndButtons(getStructuredSelection());
	}
	
	protected void doAdd()
	{
		if(addAction != null)
			addAction.run();
	}
	
	protected void doInsert()
	{
		if(insertAction != null)
			insertAction.run();
	}

	protected void doRemove()
	{
		if(removeAction != null)
			removeAction.run();
	}

	protected void doMoveUp()
	{
		if(moveUpAction != null)
			moveUpAction.run();
	}

	protected void doMoveDown()
	{
		if(moveDownAction != null)
			moveDownAction.run();
	}

	/**
	 * @see org.eclipse.update.ui.forms.internal.FormSection#doGlobalAction(java.lang.String)
	 */
	public boolean doGlobalAction(String actionId)
	{
		if (actionId.equals(ActionFactory.DELETE.getId()))
		{
			if(removeAction != null)
				removeAction.run();
			return true;
		}
		return false;
	}

	/**
	 * @see org.eclipse.hyades.test.ui.internal.editor.form.base.BaseStructuredViewerSection#fillContextMenu(org.eclipse.jface.action.IMenuManager)
	 */
	protected void fillContextMenu(IMenuManager menuManager)
	{
		if(removeAction != null)
		{
			if(removeAction.isEnabled())
				removeAction.setEnabled(!isReadOnly());
			menuManager.add(removeAction);
			menuManager.add(new Separator());
		}
		
		boolean enableCollapse = false;
		TreeItem[] treeItems = getTreeViewer().getTree().getItems();
		for (int i = 0, maxi = treeItems.length; i < maxi; i++)
		{
			if(treeItems[i].getExpanded())
			{
				enableCollapse = true;
				break;
			}
		}	
		
		if(enableCollapse)
		{
			menuManager.add(collapseAllAction);
			menuManager.add(new Separator());
		}

//		if((getStructuredSelection().size() == 1) && (((ITreeContentProvider)getTreeViewer().getContentProvider())).hasChildren(getStructuredSelection().getFirstElement()))
//		{
//			drillDownAdapter.addNavigationActions(menuManager);
//		}
	}

	/**
	 * @see org.eclipse.hyades.test.ui.internal.editor.form.base.BaseStructuredViewerSection#updateActionsAndButtons(org.eclipse.jface.viewers.IStructuredSelection)
	 */
	protected void updateActionsAndButtons(IStructuredSelection structuredSelection)
	{
		if(addAction != null)
		{
			if(addAction instanceof SelectionListenerAction)
				((SelectionListenerAction)addAction).selectionChanged(structuredSelection);
			getTreePart().setButtonEnabled(0, addAction.isEnabled());
		}

		if(removeAction != null)
		{
			removeAction.selectionChanged(structuredSelection);
			getTreePart().setButtonEnabled(1, removeAction.isEnabled());
		}

		if(moveDownAction != null)
		{
			moveDownAction.selectionChanged(structuredSelection);
			getTreePart().setButtonEnabled(3, moveDownAction.isEnabled());
		}
		
		if(moveUpAction != null)
		{
			moveUpAction.selectionChanged(structuredSelection);
			getTreePart().setButtonEnabled(2, moveUpAction.isEnabled());
		}			
	}
	/**
	 * @param insertAction The insertAction to set.
	 */
	protected void setInsertAction(IAction insertAction) {
		this.insertAction = insertAction;
	}
}
